package com.sun.corba.se.impl.transport;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.pept.transport.ByteBufferPool;

public class ByteBufferPoolImpl implements ByteBufferPool
{
    private ORB itsOrb;
    private int itsByteBufferSize;
    private ArrayList itsPool;
    private int itsObjectCounter;
    private boolean debug;
    
    public ByteBufferPoolImpl(ORB itsOrb) {
        this.itsObjectCounter = 0;
        this.itsByteBufferSize = itsOrb.getORBData().getGIOPFragmentSize();
        this.itsPool = new ArrayList();
        this.itsOrb = itsOrb;
        this.debug = itsOrb.transportDebugFlag;
    }
    
    public ByteBuffer getByteBuffer(int capacity) {
        ByteBuffer byteBuffer = null;
        if (capacity <= this.itsByteBufferSize && !this.itsOrb.getORBData().disableDirectByteBufferUse()) {
            int size;
            synchronized (this.itsPool) {
                size = this.itsPool.size();
                if (size > 0) {
                    byteBuffer = (ByteBuffer)this.itsPool.remove(size - 1);
                    byteBuffer.clear();
                }
            }
            if (size <= 0) {
                byteBuffer = ByteBuffer.allocateDirect(this.itsByteBufferSize);
            }
            ++this.itsObjectCounter;
            return byteBuffer;
        }
        else {
            byteBuffer = ByteBuffer.allocate(capacity);
            return byteBuffer;
        }
    }
    
    public void releaseByteBuffer(ByteBuffer e) {
        if (e.isDirect()) {
            synchronized (this.itsPool) {
                int n = 0;
                int identityHashCode = 0;
                if (this.debug) {
                    for (int index = 0; index < this.itsPool.size() && n == 0; ++index) {
                        if (e == (ByteBuffer)this.itsPool.get(index)) {
                            n = 1;
                            identityHashCode = System.identityHashCode(e);
                        }
                    }
                }
                if (n == 0 || !this.debug) {
                    this.itsPool.add(e);
                }
                else {
                    new Throwable(Thread.currentThread().getName() + ": Duplicate ByteBuffer reference (" + identityHashCode + ")").printStackTrace(System.out);
                }
            }
            --this.itsObjectCounter;
        }
    }
    
    public int activeCount() {
        return this.itsObjectCounter;
    }
}
